<?php
/**
 * Reflect
 * Reverse-engineer classes, interfaces, traits, functions, constants, namespaces
 * and more.
 *
 * PHP version 5
 *
 * @category PHP
 * @package  PHP_Reflect
 * @author   Laurent Laville <pear@laurent-laville.org>
 * @license  http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version  GIT: $Id$
 * @link     http://php5.laurent-laville.org/reflect/
 */

namespace Bartlett;

use Bartlett\Reflect\Event\AbstractDispatcher;
use Bartlett\Reflect\Events;
use Bartlett\Reflect\Plugin\PluginManager;
use Bartlett\Reflect\PhpParser\Lexer\TokenOffsets;

use PhpParser\Parser;
use PhpParser\NodeTraverser;
use PhpParser\NodeVisitor\NameResolver;
use PhpParser\NodeVisitor;

use Symfony\Component\Finder\Finder;

/**
 * Reflect analyse your source code with the tokenizer extension.
 *
 * All data sources (archive, local or remote script) are parsable.
 * You can cache or logs results and even more. API is extensible simply.
 *
 * @category PHP
 * @package  PHP_Reflect
 * @author   Laurent Laville <pear@laurent-laville.org>
 * @license  http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version  Release: 3.0.0
 * @link     http://php5.laurent-laville.org/reflect/
 * @since    Class available since Release 2.0.0RC1
 */
class Reflect extends AbstractDispatcher
{
    private $analysers;
    private $pluginManager;
    private $dataSourceId;

    /**
     * Creates a new instance of the Reflect engine
     */
    public function __construct()
    {
        $this->analysers = array();
    }

    /**
     * Defines a plugin manager.
     *
     * @param PluginManager $manager
     *
     * @return self for fluent interface
     */
    public function setPluginManager(PluginManager $manager)
    {
        $this->pluginManager = $manager;
        return $this;
    }

    /**
     * Returns instance of the plugin manager.
     *
     * @return PluginManager
     */
    public function getPluginManager()
    {
        return $this->pluginManager;
    }

    /**
     * Adds a new analyser to get specific metrics
     *
     * @param NodeVisitor $analyser Analyser instance
     *
     * @return self for fluent interface
     */
    public function addAnalyser(NodeVisitor $analyser)
    {
        $analyser->setSubject($this);
        $this->analysers[] = $analyser;
        return $this;
    }

    /**
     * Gets the list of active analysers.
     *
     * @return array
     */
    public function getAnalysers()
    {
        return $this->analysers;
    }

    /**
     * Set the data source identifier
     *
     * @param string $id Identitier to use for the data source
     *
     * @return self for fluent interface
     */
    public function setDataSourceId($id)
    {
        $this->dataSourceId = $id;
        return $this;
    }

    /**
     * Gets identifier of the current data source
     *
     * @return string
     */
    public function getDataSourceId()
    {
        return $this->dataSourceId;
    }

    /**
     * Analyse a data source and return all analyser metrics.
     *
     * @param Finder $finder A data source finder
     *
     * @return array|boolean array of all analysers metrics, or FALSE if no parse occured
     */
    public function parse(Finder $finder)
    {
        $metrics = array();

        if (empty($this->analysers)) {
            return false;
        }

        $lexer     = new TokenOffsets();
        $parser    = new Parser($lexer);
        $traverser = new NodeTraverser;
        $traverser->addVisitor(new NameResolver);

        // attach all analysers selected
        foreach ($this->analysers as $analyser) {
            $traverser->addVisitor($analyser);
        }

        $files = array();

        // generate a data source identifier if not provided
        if (!isset($this->dataSourceId)) {
            $this->dataSourceId = sha1(serialize($finder->getIterator()));
        }

        // analyse each file of the data source
        foreach ($finder as $uri => $file) {
            $event = $this->dispatch(
                Events::PROGRESS,
                array(
                    'source'   => $this->dataSourceId,
                    'file'     => $file,
                )
            );
            $files[] = $file->getPathname();

            if (isset($event['notModified'])) {
                $tokens = @token_get_all(
                    file_get_contents($file->getPathname())
                );
                // uses cached response (AST built by PHP-Parser)
                $stmts = $event['notModified'];

            } else {
                // live request
                try {
                    $stmts = $parser->parse(
                        file_get_contents($file->getPathname())
                    );
                    $tokens = $lexer->getTokens();

                } catch (\PhpParser\Error $e) {
                    $this->dispatch(
                        Events::ERROR,
                        array(
                            'source' => $this->dataSourceId,
                            'file'   => $file,
                            'error'  => $e->getMessage()
                        )
                    );
                    continue; // skip to next file of the data source
                }
            }

            // update context for each analyser selected
            foreach ($this->analysers as $analyser) {
                $analyser->setTokens($tokens);
                $analyser->setCurrentFile($file->getPathname());
            }

            $stmts = $traverser->traverse($stmts);

            $this->dispatch(
                Events::SUCCESS,
                array(
                    'source' => $this->dataSourceId,
                    'file'   => $file,
                    'ast'    => serialize($stmts),
                )
            );
        }

        // end of parsing the data source
        $event = $this->dispatch(Events::COMPLETE, array('source' => $this->dataSourceId));
        if (isset($event['extra'])) {
            $metrics['extra'] = $event['extra'];
        }

        // list of files parsed
        $metrics['files'] = $files;

        // collect metrics of each analyser selected
        foreach ($this->analysers as $analyser) {
            $metrics = array_merge($metrics, (array)$analyser->getMetrics());
        }

        return $metrics;
    }
}
